[llvm-branch-commits] [llvm-branch] r85750 - in /llvm/branches/Apple/Leela: include/llvm/Analysis/InlineCost.h include/llvm/Transforms/Utils/BasicInliner.h include/llvm/Transforms/Utils/InlineCost.h lib/Analysis/InlineCost.cpp lib/Transforms/IPO/InlineAlways.cpp lib/Transforms/IPO/InlineSimple.cpp lib/Transforms/IPO/Inliner.cpp lib/Transforms/Utils/InlineCost.cpp

Bill Wendling isanbard at gmail.com
Sun Nov 1 14:12:57 PST 2009


Author: void
Date: Sun Nov  1 16:12:57 2009
New Revision: 85750

URL: http://llvm.org/viewvc/llvm-project?rev=85750&view=rev
Log:
$ svn merge -c 83997 https://llvm.org/svn/llvm-project/llvm/trunk
--- Merging r83997 into '.':
U    include/llvm/Transforms/Utils/InlineCost.h
U    lib/Transforms/Utils/InlineCost.cpp
$ svn merge -c 83998 https://llvm.org/svn/llvm-project/llvm/trunk
--- Merging r83998 into '.':
A    include/llvm/Analysis/InlineCost.h
U    include/llvm/Transforms/Utils/BasicInliner.h
D    include/llvm/Transforms/Utils/InlineCost.h
A    lib/Analysis/InlineCost.cpp
U    lib/Transforms/IPO/InlineAlways.cpp
U    lib/Transforms/IPO/Inliner.cpp
U    lib/Transforms/IPO/InlineSimple.cpp
$ svn merge -c 83999 https://llvm.org/svn/llvm-project/llvm/trunk
--- Merging r83999 into '.':
D    lib/Transforms/Utils/InlineCost.cpp
$ svn merge -c 84013 https://llvm.org/svn/llvm-project/llvm/trunk
--- Merging r84013 into '.':
U    include/llvm/Analysis/InlineCost.h
U    lib/Analysis/InlineCost.cpp
$ svn merge -c 84015 https://llvm.org/svn/llvm-project/llvm/trunk
--- Merging r84015 into '.':
G    lib/Analysis/InlineCost.cpp


Added:
    llvm/branches/Apple/Leela/include/llvm/Analysis/InlineCost.h
      - copied, changed from r83998, llvm/trunk/include/llvm/Analysis/InlineCost.h
    llvm/branches/Apple/Leela/lib/Analysis/InlineCost.cpp
      - copied, changed from r83998, llvm/trunk/lib/Analysis/InlineCost.cpp
Removed:
    llvm/branches/Apple/Leela/include/llvm/Transforms/Utils/InlineCost.h
    llvm/branches/Apple/Leela/lib/Transforms/Utils/InlineCost.cpp
Modified:
    llvm/branches/Apple/Leela/include/llvm/Transforms/Utils/BasicInliner.h
    llvm/branches/Apple/Leela/lib/Transforms/IPO/InlineAlways.cpp
    llvm/branches/Apple/Leela/lib/Transforms/IPO/InlineSimple.cpp
    llvm/branches/Apple/Leela/lib/Transforms/IPO/Inliner.cpp

Copied: llvm/branches/Apple/Leela/include/llvm/Analysis/InlineCost.h (from r83998, llvm/trunk/include/llvm/Analysis/InlineCost.h)
URL: http://llvm.org/viewvc/llvm-project/llvm/branches/Apple/Leela/include/llvm/Analysis/InlineCost.h?p2=llvm/branches/Apple/Leela/include/llvm/Analysis/InlineCost.h&p1=llvm/trunk/include/llvm/Analysis/InlineCost.h&r1=83998&r2=85750&rev=85750&view=diff

==============================================================================
--- llvm/trunk/include/llvm/Analysis/InlineCost.h (original)
+++ llvm/branches/Apple/Leela/include/llvm/Analysis/InlineCost.h Sun Nov  1 16:12:57 2009
@@ -28,6 +28,40 @@
   template<class PtrType, unsigned SmallSize>
   class SmallPtrSet;
 
+  // CodeMetrics - Calculate size and a few similar metrics for a set of
+  // basic blocks.
+  struct CodeMetrics {
+    /// NeverInline - True if this callee should never be inlined into a
+    /// caller.
+    bool NeverInline;
+    
+    /// usesDynamicAlloca - True if this function calls alloca (in the C sense).
+    bool usesDynamicAlloca;
+
+    /// NumInsts, NumBlocks - Keep track of how large each function is, which
+    /// is used to estimate the code size cost of inlining it.
+    unsigned NumInsts, NumBlocks;
+
+    /// NumVectorInsts - Keep track of how many instructions produce vector
+    /// values.  The inliner is being more aggressive with inlining vector
+    /// kernels.
+    unsigned NumVectorInsts;
+    
+    /// NumRets - Keep track of how many Ret instructions the block contains.
+    unsigned NumRets;
+
+    CodeMetrics() : NeverInline(false), usesDynamicAlloca(false), NumInsts(0),
+                    NumBlocks(0), NumVectorInsts(0), NumRets(0) {}
+    
+    /// analyzeBasicBlock - Add information about the specified basic block
+    /// to the current structure.
+    void analyzeBasicBlock(const BasicBlock *BB);
+
+    /// analyzeFunction - Add information about the specified function
+    /// to the current structure.
+    void analyzeFunction(Function *F);
+  };
+
   namespace InlineConstants {
     // Various magic constants used to adjust heuristics.
     const int CallPenalty = 5;
@@ -97,58 +131,32 @@
         : ConstantWeight(CWeight), AllocaWeight(AWeight) {}
     };
     
-    // RegionInfo - Calculate size and a few related metrics for a set of
-    // basic blocks.
-    struct RegionInfo {
-      /// NeverInline - True if this callee should never be inlined into a
-      /// caller.
-      bool NeverInline;
-      
-      /// usesDynamicAlloca - True if this function calls alloca (in the C sense).
-      bool usesDynamicAlloca;
-
-      /// NumInsts, NumBlocks - Keep track of how large each function is, which
-      /// is used to estimate the code size cost of inlining it.
-      unsigned NumInsts, NumBlocks;
-
-      /// NumVectorInsts - Keep track of how many instructions produce vector
-      /// values.  The inliner is being more aggressive with inlining vector
-      /// kernels.
-      unsigned NumVectorInsts;
-      
-      /// NumRets - Keep track of how many Ret instructions the block contains.
-      unsigned NumRets;
+    struct FunctionInfo {
+      CodeMetrics Metrics;
 
       /// ArgumentWeights - Each formal argument of the function is inspected to
       /// see if it is used in any contexts where making it a constant or alloca
       /// would reduce the code size.  If so, we add some value to the argument
       /// entry here.
       std::vector<ArgInfo> ArgumentWeights;
-      
-      RegionInfo() : NeverInline(false), usesDynamicAlloca(false), NumInsts(0),
-                     NumBlocks(0), NumVectorInsts(0), NumRets(0) {}
-      
-      /// analyzeBasicBlock - Add information about the specified basic block
-      /// to the current structure.
-      void analyzeBasicBlock(const BasicBlock *BB);
-
-      /// analyzeFunction - Add information about the specified function
-      /// to the current structure.
-      void analyzeFunction(Function *F);
-
+    
       /// CountCodeReductionForConstant - Figure out an approximation for how
       /// many instructions will be constant folded if the specified value is
       /// constant.
       unsigned CountCodeReductionForConstant(Value *V);
-      
+    
       /// CountCodeReductionForAlloca - Figure out an approximation of how much
       /// smaller the function will be if it is inlined into a context where an
       /// argument becomes an alloca.
       ///
       unsigned CountCodeReductionForAlloca(Value *V);
+
+      /// analyzeFunction - Add information about the specified function
+      /// to the current structure.
+      void analyzeFunction(Function *F);
     };
 
-    std::map<const Function *, RegionInfo> CachedFunctionInfo;
+    std::map<const Function *, FunctionInfo> CachedFunctionInfo;
 
   public:
 
@@ -164,7 +172,7 @@
 
     /// resetCachedFunctionInfo - erase any cached cost info for this function.
     void resetCachedCostInfo(Function* Caller) {
-      CachedFunctionInfo[Caller].NumBlocks = 0;
+      CachedFunctionInfo[Caller].Metrics.NumBlocks = 0;
     }
   };
 }

Modified: llvm/branches/Apple/Leela/include/llvm/Transforms/Utils/BasicInliner.h
URL: http://llvm.org/viewvc/llvm-project/llvm/branches/Apple/Leela/include/llvm/Transforms/Utils/BasicInliner.h?rev=85750&r1=85749&r2=85750&view=diff

==============================================================================
--- llvm/branches/Apple/Leela/include/llvm/Transforms/Utils/BasicInliner.h (original)
+++ llvm/branches/Apple/Leela/include/llvm/Transforms/Utils/BasicInliner.h Sun Nov  1 16:12:57 2009
@@ -15,7 +15,7 @@
 #ifndef BASICINLINER_H
 #define BASICINLINER_H
 
-#include "llvm/Transforms/Utils/InlineCost.h"
+#include "llvm/Analysis/InlineCost.h"
 
 namespace llvm {
 

Removed: llvm/branches/Apple/Leela/include/llvm/Transforms/Utils/InlineCost.h
URL: http://llvm.org/viewvc/llvm-project/llvm/branches/Apple/Leela/include/llvm/Transforms/Utils/InlineCost.h?rev=85749&view=auto

==============================================================================
--- llvm/branches/Apple/Leela/include/llvm/Transforms/Utils/InlineCost.h (original)
+++ llvm/branches/Apple/Leela/include/llvm/Transforms/Utils/InlineCost.h (removed)
@@ -1,164 +0,0 @@
-//===- InlineCost.cpp - Cost analysis for inliner ---------------*- C++ -*-===//
-//
-//                     The LLVM Compiler Infrastructure
-//
-// This file is distributed under the University of Illinois Open Source
-// License. See LICENSE.TXT for details.
-//
-//===----------------------------------------------------------------------===//
-//
-// This file implements heuristics for inlining decisions.
-//
-//===----------------------------------------------------------------------===//
-
-#ifndef LLVM_TRANSFORMS_UTILS_INLINECOST_H
-#define LLVM_TRANSFORMS_UTILS_INLINECOST_H
-
-#include <cassert>
-#include <climits>
-#include <map>
-#include <vector>
-
-namespace llvm {
-
-  class Value;
-  class Function;
-  class CallSite;
-  template<class PtrType, unsigned SmallSize>
-  class SmallPtrSet;
-
-  namespace InlineConstants {
-    // Various magic constants used to adjust heuristics.
-    const int CallPenalty = 5;
-    const int LastCallToStaticBonus = -15000;
-    const int ColdccPenalty = 2000;
-    const int NoreturnPenalty = 10000;
-  }
-
-  /// InlineCost - Represent the cost of inlining a function. This
-  /// supports special values for functions which should "always" or
-  /// "never" be inlined. Otherwise, the cost represents a unitless
-  /// amount; smaller values increase the likelyhood of the function
-  /// being inlined.
-  class InlineCost {
-    enum Kind {
-      Value,
-      Always,
-      Never
-    };
-
-    // This is a do-it-yourself implementation of
-    //   int Cost : 30;
-    //   unsigned Type : 2;
-    // We used to use bitfields, but they were sometimes miscompiled (PR3822).
-    enum { TYPE_BITS = 2 };
-    enum { COST_BITS = unsigned(sizeof(unsigned)) * CHAR_BIT - TYPE_BITS };
-    unsigned TypedCost; // int Cost : COST_BITS; unsigned Type : TYPE_BITS;
-
-    Kind getType() const {
-      return Kind(TypedCost >> COST_BITS);
-    }
-
-    int getCost() const {
-      // Sign-extend the bottom COST_BITS bits.
-      return (int(TypedCost << TYPE_BITS)) >> TYPE_BITS;
-    }
-
-    InlineCost(int C, int T) {
-      TypedCost = (unsigned(C << TYPE_BITS) >> TYPE_BITS) | (T << COST_BITS);
-      assert(getCost() == C && "Cost exceeds InlineCost precision");
-    }
-  public:
-    static InlineCost get(int Cost) { return InlineCost(Cost, Value); }
-    static InlineCost getAlways() { return InlineCost(0, Always); }
-    static InlineCost getNever() { return InlineCost(0, Never); }
-
-    bool isVariable() const { return getType() == Value; }
-    bool isAlways() const { return getType() == Always; }
-    bool isNever() const { return getType() == Never; }
-
-    /// getValue() - Return a "variable" inline cost's amount. It is
-    /// an error to call this on an "always" or "never" InlineCost.
-    int getValue() const {
-      assert(getType() == Value && "Invalid access of InlineCost");
-      return getCost();
-    }
-  };
-  
-  /// InlineCostAnalyzer - Cost analyzer used by inliner.
-  class InlineCostAnalyzer {
-    struct ArgInfo {
-    public:
-      unsigned ConstantWeight;
-      unsigned AllocaWeight;
-      
-      ArgInfo(unsigned CWeight, unsigned AWeight)
-        : ConstantWeight(CWeight), AllocaWeight(AWeight) {}
-    };
-    
-    // FunctionInfo - For each function, calculate the size of it in blocks and
-    // instructions.
-    struct FunctionInfo {
-      /// NeverInline - True if this callee should never be inlined into a
-      /// caller.
-      bool NeverInline;
-      
-      /// usesDynamicAlloca - True if this function calls alloca (in the C sense).
-      bool usesDynamicAlloca;
-
-      /// NumInsts, NumBlocks - Keep track of how large each function is, which
-      /// is used to estimate the code size cost of inlining it.
-      unsigned NumInsts, NumBlocks;
-
-      /// NumVectorInsts - Keep track of how many instructions produce vector
-      /// values.  The inliner is being more aggressive with inlining vector
-      /// kernels.
-      unsigned NumVectorInsts;
-      
-      /// ArgumentWeights - Each formal argument of the function is inspected to
-      /// see if it is used in any contexts where making it a constant or alloca
-      /// would reduce the code size.  If so, we add some value to the argument
-      /// entry here.
-      std::vector<ArgInfo> ArgumentWeights;
-      
-      FunctionInfo() : NeverInline(false), usesDynamicAlloca(false), NumInsts(0),
-                       NumBlocks(0), NumVectorInsts(0) {}
-      
-      /// analyzeFunction - Fill in the current structure with information
-      /// gleaned from the specified function.
-      void analyzeFunction(Function *F);
-
-      /// CountCodeReductionForConstant - Figure out an approximation for how
-      /// many instructions will be constant folded if the specified value is
-      /// constant.
-      unsigned CountCodeReductionForConstant(Value *V);
-      
-      /// CountCodeReductionForAlloca - Figure out an approximation of how much
-      /// smaller the function will be if it is inlined into a context where an
-      /// argument becomes an alloca.
-      ///
-      unsigned CountCodeReductionForAlloca(Value *V);
-    };
-
-    std::map<const Function *, FunctionInfo> CachedFunctionInfo;
-
-  public:
-
-    /// getInlineCost - The heuristic used to determine if we should inline the
-    /// function call or not.
-    ///
-    InlineCost getInlineCost(CallSite CS,
-                             SmallPtrSet<const Function *, 16> &NeverInline);
-
-    /// getInlineFudgeFactor - Return a > 1.0 factor if the inliner should use a
-    /// higher threshold to determine if the function call should be inlined.
-    float getInlineFudgeFactor(CallSite CS);
-
-    /// resetCachedFunctionInfo - erase any cached cost info for this function.
-    void resetCachedCostInfo(Function* Caller) {
-      CachedFunctionInfo[Caller].NumBlocks = 0;
-    }
-  };
-}
-
-#endif

Copied: llvm/branches/Apple/Leela/lib/Analysis/InlineCost.cpp (from r83998, llvm/trunk/lib/Analysis/InlineCost.cpp)
URL: http://llvm.org/viewvc/llvm-project/llvm/branches/Apple/Leela/lib/Analysis/InlineCost.cpp?p2=llvm/branches/Apple/Leela/lib/Analysis/InlineCost.cpp&p1=llvm/trunk/lib/Analysis/InlineCost.cpp&r1=83998&r2=85750&rev=85750&view=diff

==============================================================================
--- llvm/branches/Apple/Leela/lib/Transforms/IPO/InlineAlways.cpp (original)
+++ llvm/branches/Apple/Leela/lib/Transforms/IPO/InlineAlways.cpp Sun Nov  1 16:12:57 2009
@@ -19,11 +19,11 @@
 #include "llvm/Module.h"
 #include "llvm/Type.h"
 #include "llvm/Analysis/CallGraph.h"
+#include "llvm/Analysis/InlineCost.h"
 #include "llvm/Support/CallSite.h"
 #include "llvm/Support/Compiler.h"
 #include "llvm/Transforms/IPO.h"
 #include "llvm/Transforms/IPO/InlinerPass.h"
-#include "llvm/Transforms/Utils/InlineCost.h"
 #include "llvm/ADT/SmallPtrSet.h"
 
 using namespace llvm;

Modified: llvm/branches/Apple/Leela/lib/Transforms/IPO/InlineSimple.cpp
URL: http://llvm.org/viewvc/llvm-project/llvm/branches/Apple/Leela/lib/Transforms/IPO/InlineSimple.cpp?rev=85750&r1=85749&r2=85750&view=diff

==============================================================================
--- llvm/branches/Apple/Leela/lib/Transforms/IPO/InlineSimple.cpp (original)
+++ llvm/branches/Apple/Leela/lib/Transforms/IPO/InlineSimple.cpp Sun Nov  1 16:12:57 2009
@@ -18,11 +18,11 @@
 #include "llvm/Module.h"
 #include "llvm/Type.h"
 #include "llvm/Analysis/CallGraph.h"
+#include "llvm/Analysis/InlineCost.h"
 #include "llvm/Support/CallSite.h"
 #include "llvm/Support/Compiler.h"
 #include "llvm/Transforms/IPO.h"
 #include "llvm/Transforms/IPO/InlinerPass.h"
-#include "llvm/Transforms/Utils/InlineCost.h"
 #include "llvm/ADT/SmallPtrSet.h"
 
 using namespace llvm;

Modified: llvm/branches/Apple/Leela/lib/Transforms/IPO/Inliner.cpp
URL: http://llvm.org/viewvc/llvm-project/llvm/branches/Apple/Leela/lib/Transforms/IPO/Inliner.cpp?rev=85750&r1=85749&r2=85750&view=diff

==============================================================================
--- llvm/branches/Apple/Leela/lib/Transforms/IPO/Inliner.cpp (original)
+++ llvm/branches/Apple/Leela/lib/Transforms/IPO/Inliner.cpp Sun Nov  1 16:12:57 2009
@@ -18,10 +18,10 @@
 #include "llvm/Instructions.h"
 #include "llvm/IntrinsicInst.h"
 #include "llvm/Analysis/CallGraph.h"
+#include "llvm/Analysis/InlineCost.h"
 #include "llvm/Support/CallSite.h"
 #include "llvm/Target/TargetData.h"
 #include "llvm/Transforms/IPO/InlinerPass.h"
-#include "llvm/Transforms/Utils/InlineCost.h"
 #include "llvm/Transforms/Utils/Cloning.h"
 #include "llvm/Support/CommandLine.h"
 #include "llvm/Support/Debug.h"

Removed: llvm/branches/Apple/Leela/lib/Transforms/Utils/InlineCost.cpp
URL: http://llvm.org/viewvc/llvm-project/llvm/branches/Apple/Leela/lib/Transforms/Utils/InlineCost.cpp?rev=85749&view=auto

==============================================================================
--- llvm/branches/Apple/Leela/lib/Transforms/Utils/InlineCost.cpp (original)
+++ llvm/branches/Apple/Leela/lib/Transforms/Utils/InlineCost.cpp (removed)
@@ -1,337 +0,0 @@
-//===- InlineCost.cpp - Cost analysis for inliner -------------------------===//
-//
-//                     The LLVM Compiler Infrastructure
-//
-// This file is distributed under the University of Illinois Open Source
-// License. See LICENSE.TXT for details.
-//
-//===----------------------------------------------------------------------===//
-//
-// This file implements inline cost analysis.
-//
-//===----------------------------------------------------------------------===//
-
-#include "llvm/Transforms/Utils/InlineCost.h"
-#include "llvm/Support/CallSite.h"
-#include "llvm/CallingConv.h"
-#include "llvm/IntrinsicInst.h"
-#include "llvm/ADT/SmallPtrSet.h"
-using namespace llvm;
-
-// CountCodeReductionForConstant - Figure out an approximation for how many
-// instructions will be constant folded if the specified value is constant.
-//
-unsigned InlineCostAnalyzer::FunctionInfo::
-         CountCodeReductionForConstant(Value *V) {
-  unsigned Reduction = 0;
-  for (Value::use_iterator UI = V->use_begin(), E = V->use_end(); UI != E; ++UI)
-    if (isa<BranchInst>(*UI))
-      Reduction += 40;          // Eliminating a conditional branch is a big win
-    else if (SwitchInst *SI = dyn_cast<SwitchInst>(*UI))
-      // Eliminating a switch is a big win, proportional to the number of edges
-      // deleted.
-      Reduction += (SI->getNumSuccessors()-1) * 40;
-    else if (CallInst *CI = dyn_cast<CallInst>(*UI)) {
-      // Turning an indirect call into a direct call is a BIG win
-      Reduction += CI->getCalledValue() == V ? 500 : 0;
-    } else if (InvokeInst *II = dyn_cast<InvokeInst>(*UI)) {
-      // Turning an indirect call into a direct call is a BIG win
-      Reduction += II->getCalledValue() == V ? 500 : 0;
-    } else {
-      // Figure out if this instruction will be removed due to simple constant
-      // propagation.
-      Instruction &Inst = cast<Instruction>(**UI);
-      
-      // We can't constant propagate instructions which have effects or
-      // read memory.
-      //
-      // FIXME: It would be nice to capture the fact that a load from a
-      // pointer-to-constant-global is actually a *really* good thing to zap.
-      // Unfortunately, we don't know the pointer that may get propagated here,
-      // so we can't make this decision.
-      if (Inst.mayReadFromMemory() || Inst.mayHaveSideEffects() ||
-          isa<AllocationInst>(Inst)) 
-        continue;
-
-      bool AllOperandsConstant = true;
-      for (unsigned i = 0, e = Inst.getNumOperands(); i != e; ++i)
-        if (!isa<Constant>(Inst.getOperand(i)) && Inst.getOperand(i) != V) {
-          AllOperandsConstant = false;
-          break;
-        }
-
-      if (AllOperandsConstant) {
-        // We will get to remove this instruction...
-        Reduction += 7;
-
-        // And any other instructions that use it which become constants
-        // themselves.
-        Reduction += CountCodeReductionForConstant(&Inst);
-      }
-    }
-
-  return Reduction;
-}
-
-// CountCodeReductionForAlloca - Figure out an approximation of how much smaller
-// the function will be if it is inlined into a context where an argument
-// becomes an alloca.
-//
-unsigned InlineCostAnalyzer::FunctionInfo::
-         CountCodeReductionForAlloca(Value *V) {
-  if (!isa<PointerType>(V->getType())) return 0;  // Not a pointer
-  unsigned Reduction = 0;
-  for (Value::use_iterator UI = V->use_begin(), E = V->use_end(); UI != E;++UI){
-    Instruction *I = cast<Instruction>(*UI);
-    if (isa<LoadInst>(I) || isa<StoreInst>(I))
-      Reduction += 10;
-    else if (GetElementPtrInst *GEP = dyn_cast<GetElementPtrInst>(I)) {
-      // If the GEP has variable indices, we won't be able to do much with it.
-      if (!GEP->hasAllConstantIndices())
-        Reduction += CountCodeReductionForAlloca(GEP)+15;
-    } else {
-      // If there is some other strange instruction, we're not going to be able
-      // to do much if we inline this.
-      return 0;
-    }
-  }
-
-  return Reduction;
-}
-
-/// analyzeFunction - Fill in the current structure with information gleaned
-/// from the specified function.
-void InlineCostAnalyzer::FunctionInfo::analyzeFunction(Function *F) {
-  unsigned NumInsts = 0, NumBlocks = 0, NumVectorInsts = 0, NumRets = 0;
-
-  // Look at the size of the callee.  Each basic block counts as 20 units, and
-  // each instruction counts as 5.
-  for (Function::const_iterator BB = F->begin(), E = F->end(); BB != E; ++BB) {
-    for (BasicBlock::const_iterator II = BB->begin(), E = BB->end();
-         II != E; ++II) {
-      if (isa<PHINode>(II)) continue;           // PHI nodes don't count.
-
-      // Special handling for calls.
-      if (isa<CallInst>(II) || isa<InvokeInst>(II)) {
-        if (isa<DbgInfoIntrinsic>(II))
-          continue;  // Debug intrinsics don't count as size.
-        
-        CallSite CS = CallSite::get(const_cast<Instruction*>(&*II));
-        
-        // If this function contains a call to setjmp or _setjmp, never inline
-        // it.  This is a hack because we depend on the user marking their local
-        // variables as volatile if they are live across a setjmp call, and they
-        // probably won't do this in callers.
-        if (Function *F = CS.getCalledFunction())
-          if (F->isDeclaration() && 
-              (F->getName() == "setjmp" || F->getName() == "_setjmp")) {
-            NeverInline = true;
-            return;
-          }
-
-        // Calls often compile into many machine instructions.  Bump up their
-        // cost to reflect this.
-        if (!isa<IntrinsicInst>(II))
-          NumInsts += InlineConstants::CallPenalty;
-      }
-      
-      // These, too, are calls.
-      if (isa<MallocInst>(II) || isa<FreeInst>(II))
-	NumInsts += InlineConstants::CallPenalty;
-
-      if (const AllocaInst *AI = dyn_cast<AllocaInst>(II)) {
-        if (!AI->isStaticAlloca())
-          this->usesDynamicAlloca = true;
-      }
-
-      if (isa<ExtractElementInst>(II) || isa<VectorType>(II->getType()))
-        ++NumVectorInsts; 
-      
-      // Noop casts, including ptr <-> int,  don't count.
-      if (const CastInst *CI = dyn_cast<CastInst>(II)) {
-        if (CI->isLosslessCast() || isa<IntToPtrInst>(CI) || 
-            isa<PtrToIntInst>(CI))
-          continue;
-      } else if (const GetElementPtrInst *GEPI =
-                 dyn_cast<GetElementPtrInst>(II)) {
-        // If a GEP has all constant indices, it will probably be folded with
-        // a load/store.
-        if (GEPI->hasAllConstantIndices())
-          continue;
-      }
-
-      if (isa<ReturnInst>(II))
-        ++NumRets;
-      
-      ++NumInsts;
-    }
-
-    ++NumBlocks;
-  }
-
-  // A function with exactly one return has it removed during the inlining
-  // process (see InlineFunction), so don't count it.
-  if (NumRets==1)
-    --NumInsts;
-
-  this->NumBlocks      = NumBlocks;
-  this->NumInsts       = NumInsts;
-  this->NumVectorInsts = NumVectorInsts;
-
-  // Check out all of the arguments to the function, figuring out how much
-  // code can be eliminated if one of the arguments is a constant.
-  for (Function::arg_iterator I = F->arg_begin(), E = F->arg_end(); I != E; ++I)
-    ArgumentWeights.push_back(ArgInfo(CountCodeReductionForConstant(I),
-                                      CountCodeReductionForAlloca(I)));
-}
-
-
-
-// getInlineCost - The heuristic used to determine if we should inline the
-// function call or not.
-//
-InlineCost InlineCostAnalyzer::getInlineCost(CallSite CS,
-                               SmallPtrSet<const Function *, 16> &NeverInline) {
-  Instruction *TheCall = CS.getInstruction();
-  Function *Callee = CS.getCalledFunction();
-  Function *Caller = TheCall->getParent()->getParent();
-
-  // Don't inline functions which can be redefined at link-time to mean
-  // something else.  Don't inline functions marked noinline.
-  if (Callee->mayBeOverridden() ||
-      Callee->hasFnAttr(Attribute::NoInline) || NeverInline.count(Callee))
-    return llvm::InlineCost::getNever();
-
-  // InlineCost - This value measures how good of an inline candidate this call
-  // site is to inline.  A lower inline cost make is more likely for the call to
-  // be inlined.  This value may go negative.
-  //
-  int InlineCost = 0;
-  
-  // If there is only one call of the function, and it has internal linkage,
-  // make it almost guaranteed to be inlined.
-  //
-  if (Callee->hasLocalLinkage() && Callee->hasOneUse())
-    InlineCost += InlineConstants::LastCallToStaticBonus;
-  
-  // If this function uses the coldcc calling convention, prefer not to inline
-  // it.
-  if (Callee->getCallingConv() == CallingConv::Cold)
-    InlineCost += InlineConstants::ColdccPenalty;
-  
-  // If the instruction after the call, or if the normal destination of the
-  // invoke is an unreachable instruction, the function is noreturn.  As such,
-  // there is little point in inlining this.
-  if (InvokeInst *II = dyn_cast<InvokeInst>(TheCall)) {
-    if (isa<UnreachableInst>(II->getNormalDest()->begin()))
-      InlineCost += InlineConstants::NoreturnPenalty;
-  } else if (isa<UnreachableInst>(++BasicBlock::iterator(TheCall)))
-    InlineCost += InlineConstants::NoreturnPenalty;
-  
-  // Get information about the callee...
-  FunctionInfo &CalleeFI = CachedFunctionInfo[Callee];
-  
-  // If we haven't calculated this information yet, do so now.
-  if (CalleeFI.NumBlocks == 0)
-    CalleeFI.analyzeFunction(Callee);
-
-  // If we should never inline this, return a huge cost.
-  if (CalleeFI.NeverInline)
-    return InlineCost::getNever();
-
-  // FIXME: It would be nice to kill off CalleeFI.NeverInline. Then we
-  // could move this up and avoid computing the FunctionInfo for
-  // things we are going to just return always inline for. This
-  // requires handling setjmp somewhere else, however.
-  if (!Callee->isDeclaration() && Callee->hasFnAttr(Attribute::AlwaysInline))
-    return InlineCost::getAlways();
-    
-  if (CalleeFI.usesDynamicAlloca) {
-    // Get infomation about the caller...
-    FunctionInfo &CallerFI = CachedFunctionInfo[Caller];
-
-    // If we haven't calculated this information yet, do so now.
-    if (CallerFI.NumBlocks == 0)
-      CallerFI.analyzeFunction(Caller);
-
-    // Don't inline a callee with dynamic alloca into a caller without them.
-    // Functions containing dynamic alloca's are inefficient in various ways;
-    // don't create more inefficiency.
-    if (!CallerFI.usesDynamicAlloca)
-      return InlineCost::getNever();
-  }
-
-  // Add to the inline quality for properties that make the call valuable to
-  // inline.  This includes factors that indicate that the result of inlining
-  // the function will be optimizable.  Currently this just looks at arguments
-  // passed into the function.
-  //
-  unsigned ArgNo = 0;
-  for (CallSite::arg_iterator I = CS.arg_begin(), E = CS.arg_end();
-       I != E; ++I, ++ArgNo) {
-    // Each argument passed in has a cost at both the caller and the callee
-    // sides.  This favors functions that take many arguments over functions
-    // that take few arguments.
-    InlineCost -= 20;
-    
-    // If this is a function being passed in, it is very likely that we will be
-    // able to turn an indirect function call into a direct function call.
-    if (isa<Function>(I))
-      InlineCost -= 100;
-    
-    // If an alloca is passed in, inlining this function is likely to allow
-    // significant future optimization possibilities (like scalar promotion, and
-    // scalarization), so encourage the inlining of the function.
-    //
-    else if (isa<AllocaInst>(I)) {
-      if (ArgNo < CalleeFI.ArgumentWeights.size())
-        InlineCost -= CalleeFI.ArgumentWeights[ArgNo].AllocaWeight;
-      
-      // If this is a constant being passed into the function, use the argument
-      // weights calculated for the callee to determine how much will be folded
-      // away with this information.
-    } else if (isa<Constant>(I)) {
-      if (ArgNo < CalleeFI.ArgumentWeights.size())
-        InlineCost -= CalleeFI.ArgumentWeights[ArgNo].ConstantWeight;
-    }
-  }
-  
-  // Now that we have considered all of the factors that make the call site more
-  // likely to be inlined, look at factors that make us not want to inline it.
-  
-  // Don't inline into something too big, which would make it bigger.
-  // "size" here is the number of basic blocks, not instructions.
-  //
-  InlineCost += Caller->size()/15;
-  
-  // Look at the size of the callee. Each instruction counts as 5.
-  InlineCost += CalleeFI.NumInsts*5;
-
-  return llvm::InlineCost::get(InlineCost);
-}
-
-// getInlineFudgeFactor - Return a > 1.0 factor if the inliner should use a
-// higher threshold to determine if the function call should be inlined.
-float InlineCostAnalyzer::getInlineFudgeFactor(CallSite CS) {
-  Function *Callee = CS.getCalledFunction();
-  
-  // Get information about the callee...
-  FunctionInfo &CalleeFI = CachedFunctionInfo[Callee];
-  
-  // If we haven't calculated this information yet, do so now.
-  if (CalleeFI.NumBlocks == 0)
-    CalleeFI.analyzeFunction(Callee);
-
-  float Factor = 1.0f;
-  // Single BB functions are often written to be inlined.
-  if (CalleeFI.NumBlocks == 1)
-    Factor += 0.5f;
-
-  // Be more aggressive if the function contains a good chunk (if it mades up
-  // at least 10% of the instructions) of vector instructions.
-  if (CalleeFI.NumVectorInsts > CalleeFI.NumInsts/2)
-    Factor += 2.0f;
-  else if (CalleeFI.NumVectorInsts > CalleeFI.NumInsts/10)
-    Factor += 1.5f;
-  return Factor;
-}





More information about the llvm-branch-commits mailing list
--- llvm/trunk/lib/Analysis/InlineCost.cpp (original)
+++ llvm/branches/Apple/Leela/lib/Analysis/InlineCost.cpp Sun Nov  1 16:12:57 2009
@@ -21,7 +21,7 @@
 // CountCodeReductionForConstant - Figure out an approximation for how many
 // instructions will be constant folded if the specified value is constant.
 //
-unsigned InlineCostAnalyzer::RegionInfo::
+unsigned InlineCostAnalyzer::FunctionInfo::
          CountCodeReductionForConstant(Value *V) {
   unsigned Reduction = 0;
   for (Value::use_iterator UI = V->use_begin(), E = V->use_end(); UI != E; ++UI)
@@ -77,7 +77,7 @@
 // the function will be if it is inlined into a context where an argument
 // becomes an alloca.
 //
-unsigned InlineCostAnalyzer::RegionInfo::
+unsigned InlineCostAnalyzer::FunctionInfo::
          CountCodeReductionForAlloca(Value *V) {
   if (!isa<PointerType>(V->getType())) return 0;  // Not a pointer
   unsigned Reduction = 0;
@@ -101,7 +101,7 @@
 
 /// analyzeBasicBlock - Fill in the current structure with information gleaned
 /// from the specified block.
-void InlineCostAnalyzer::RegionInfo::analyzeBasicBlock(const BasicBlock *BB) {
+void CodeMetrics::analyzeBasicBlock(const BasicBlock *BB) {
   ++NumBlocks;
 
   for (BasicBlock::const_iterator II = BB->begin(), E = BB->end();
@@ -121,10 +121,8 @@
       // probably won't do this in callers.
       if (Function *F = CS.getCalledFunction())
         if (F->isDeclaration() && 
-            (F->getName() == "setjmp" || F->getName() == "_setjmp")) {
+            (F->getName() == "setjmp" || F->getName() == "_setjmp"))
           NeverInline = true;
-          return;
-        }
 
       // Calls often compile into many machine instructions.  Bump up their
       // cost to reflect this.
@@ -166,17 +164,22 @@
 
 /// analyzeFunction - Fill in the current structure with information gleaned
 /// from the specified function.
-void InlineCostAnalyzer::RegionInfo::analyzeFunction(Function *F) {
-  // Look at the size of the callee.  Each basic block counts as 20 units, and
-  // each instruction counts as 5.
+void CodeMetrics::analyzeFunction(Function *F) {
+  // Look at the size of the callee.
   for (Function::const_iterator BB = F->begin(), E = F->end(); BB != E; ++BB)
     analyzeBasicBlock(&*BB);
+}
+
+/// analyzeFunction - Fill in the current structure with information gleaned
+/// from the specified function.
+void InlineCostAnalyzer::FunctionInfo::analyzeFunction(Function *F) {
+  Metrics.analyzeFunction(F);
 
   // A function with exactly one return has it removed during the inlining
   // process (see InlineFunction), so don't count it.
-  // FIXME: This knowledge should really be encoded outside of RegionInfo.
-  if (NumRets==1)
-    --NumInsts;
+  // FIXME: This knowledge should really be encoded outside of FunctionInfo.
+  if (Metrics.NumRets==1)
+    --Metrics.NumInsts;
 
   // Check out all of the arguments to the function, figuring out how much
   // code can be eliminated if one of the arguments is a constant.
@@ -185,8 +188,6 @@
                                       CountCodeReductionForAlloca(I)));
 }
 
-
-
 // getInlineCost - The heuristic used to determine if we should inline the
 // function call or not.
 //
@@ -229,35 +230,35 @@
     InlineCost += InlineConstants::NoreturnPenalty;
   
   // Get information about the callee...
-  RegionInfo &CalleeFI = CachedFunctionInfo[Callee];
+  FunctionInfo &CalleeFI = CachedFunctionInfo[Callee];
   
   // If we haven't calculated this information yet, do so now.
-  if (CalleeFI.NumBlocks == 0)
+  if (CalleeFI.Metrics.NumBlocks == 0)
     CalleeFI.analyzeFunction(Callee);
 
   // If we should never inline this, return a huge cost.
-  if (CalleeFI.NeverInline)
+  if (CalleeFI.Metrics.NeverInline)
     return InlineCost::getNever();
 
   // FIXME: It would be nice to kill off CalleeFI.NeverInline. Then we
   // could move this up and avoid computing the RegionInfo for
   // things we are going to just return always inline for. This
   // requires handling setjmp somewhere else, however.
   if (!Callee->isDeclaration() && Callee->hasFnAttr(Attribute::AlwaysInline))
     return InlineCost::getAlways();
     
-  if (CalleeFI.usesDynamicAlloca) {
+  if (CalleeFI.Metrics.usesDynamicAlloca) {
     // Get infomation about the caller...
-    RegionInfo &CallerFI = CachedFunctionInfo[Caller];
+    FunctionInfo &CallerFI = CachedFunctionInfo[Caller];
 
     // If we haven't calculated this information yet, do so now.
-    if (CallerFI.NumBlocks == 0)
+    if (CallerFI.Metrics.NumBlocks == 0)
       CallerFI.analyzeFunction(Caller);
 
     // Don't inline a callee with dynamic alloca into a caller without them.
     // Functions containing dynamic alloca's are inefficient in various ways;
     // don't create more inefficiency.
-    if (!CallerFI.usesDynamicAlloca)
+    if (!CallerFI.Metrics.usesDynamicAlloca)
       return InlineCost::getNever();
   }
 
@@ -305,7 +306,7 @@
   InlineCost += Caller->size()/15;
   
   // Look at the size of the callee. Each instruction counts as 5.
-  InlineCost += CalleeFI.NumInsts*5;
+  InlineCost += CalleeFI.Metrics.NumInsts*5;
 
   return llvm::InlineCost::get(InlineCost);
 }
@@ -316,22 +317,22 @@
   Function *Callee = CS.getCalledFunction();
   
   // Get information about the callee...
-  RegionInfo &CalleeFI = CachedFunctionInfo[Callee];
+  FunctionInfo &CalleeFI = CachedFunctionInfo[Callee];
   
   // If we haven't calculated this information yet, do so now.
-  if (CalleeFI.NumBlocks == 0)
+  if (CalleeFI.Metrics.NumBlocks == 0)
     CalleeFI.analyzeFunction(Callee);
 
   float Factor = 1.0f;
   // Single BB functions are often written to be inlined.
-  if (CalleeFI.NumBlocks == 1)
+  if (CalleeFI.Metrics.NumBlocks == 1)
     Factor += 0.5f;
 
   // Be more aggressive if the function contains a good chunk (if it mades up
   // at least 10% of the instructions) of vector instructions.
-  if (CalleeFI.NumVectorInsts > CalleeFI.NumInsts/2)
+  if (CalleeFI.Metrics.NumVectorInsts > CalleeFI.Metrics.NumInsts/2)
     Factor += 2.0f;
-  else if (CalleeFI.NumVectorInsts > CalleeFI.NumInsts/10)
+  else if (CalleeFI.Metrics.NumVectorInsts > CalleeFI.Metrics.NumInsts/10)
     Factor += 1.5f;
   return Factor;
 }
@@ -229,35 +230,35 @@
 
   // FIXME: It would be nice to kill off CalleeFI.NeverInline. Then we
-  // could move this up and avoid computing the RegionInfo for
+  // could move this up and avoid computing the FunctionInfo for
   // things we are going to just return always inline for. This
   // requires handling setjmp somewhere else, however.

Modified: llvm/branches/Apple/Leela/lib/Transforms/IPO/InlineAlways.cpp
URL: http://llvm.org/viewvc/llvm-project/llvm/branches/Apple/Leela/lib/Transforms/IPO/InlineAlways.cpp?rev=85750&r1=85749&r2=85750&view=diff

==============================================================================